require_relative 'mail_daemon/version'
require_relative 'mail_daemon/helpers'
require_relative 'mail_daemon/email_body_parser'
require_relative 'mail_daemon/email_handler'
require_relative 'mail_daemon/imap/statuses'
require_relative 'mail_daemon/imap/connection'
require_relative 'mail_daemon/imap/watcher'
require_relative 'mail_daemon/encryption'
require 'sinatra/base'
require 'json'
require 'open3'
require 'mail'

include MailDaemon::Helpers

module MailDaemon
  class Handler
    def initialize(options)
      setup_options(options)
      default_option :connections, []
      default_option :debug, false

      puts "Setting up Signal Traps" if @options[:debug]
      Signal.trap("INT") {
        Thread.new {self.stop}.join
      }
      # Trap `Kill `
      Signal.trap("TERM") {
        Thread.new {self.stop}.join
      }
    end

    def restart
      puts "Restarting... " if @options[:debug]
      self.stop if self.running?
      Thread.new do
        puts "Starting in new thread " if @options[:debug]
        self.start(&@client_handler_block)
      end
    end

    def running?
      @watchers && @watchers.detect{|watcher| watcher.running?}
    end

    # Signal catching
    def stop
      @watchers.map{|watcher| watcher.stop}
      @threads.map{|t| t.kill }
    end

    def reload(options)
      reload_options(options)
      self.restart
    end

    def start(&block)
      @watchers = []

      @client_handler_block = block

      @options[:connections].each do |mailbox|
        mailbox[:ssl_options] = {:verify_mode => OpenSSL::SSL::VERIFY_NONE} if mailbox[:ssl]

        puts "Setting up watcher for #{mailbox[:username]}" if @options[:debug]
        @watchers << MailDaemon::Imap::Watcher.new(@options.merge(mailbox))
      end
      @threads = []
      @watchers.each do |watcher|
        @threads << Thread.new do
          begin
            watcher.start do |message|
              type = message.delete(:type)
              yield message, type
            end
          rescue => e
            puts "#{watcher}: #{e.message}" 
          end
        end
      end

      @threads.map{|t| t.join}

    end

  end
end
